package no.ntnu.idatt1002.demo.data.Economics;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * ItemRegister is a generic class used
 * for storing either Income or Expense.
 *
 * @param <T> Income or Expense
 */
public class ItemRegister<T extends Item>{
    List<T> items;

    /**
     * An "empty" class constructor.
     */
    public ItemRegister() {
        this.items = new ArrayList<>();
    }

    /**
     * Class constructor that takes in a List of T=(Income or Expense) as argument.
     *
     * @param items a List of T=(Income or Expense) you want to register.
     */
    public ItemRegister(List<T> items) {
        this.items = items;
    }

    /**
     * Get a List of every T=(Income or Expense).
     * @return T=(Income or Expense) List.
     */
    public List<T> getItems() {
        return items;
    }

    /**
     * Add a new T=(Income or Expense) to the register. Throws IllegalArgumentException
     * if the new T=(Income or Expense) is already registered.
     *
     * @param newItem the T=(Income or Expense) you want to add.
     * @throws IllegalArgumentException if the item is already in the register.
     */
    public void addItem(T newItem) throws IllegalArgumentException{
        if(items.contains(newItem)){
            throw new IllegalArgumentException("This item is already registered");
        }
        items.add(newItem);
    }

    /**
     * Get the sum of all T´s=(Income or Expenses) in items.
     *
     * @return sum of all the T´s=(Income or Expenses).
     */
    public double getTotalSum(){
        return items.stream().map(Item::getAmount).mapToDouble(Double::doubleValue).sum();
    }

    /**
     * Get a List of every T=(Income or Expense)
     * in a given category.
     *
     * @param category the category you want to get every T=(Income or Expense) of.
     * @param <S> IncomeCategory or ExpenseCategory
     * @throws IllegalArgumentException if S is not IncomeCategory or ExpenseCategory
     * @throws ClassCastException if you use an IncomeCategory for Expense register (and vice versa)
     * @return a List of every T=(Income or Expense) in S=(IncomeCategory or ExpenseCategory)
     */
    public <S> List<T> getItemsByCategory(S category) {
        //Checks if category is an instance of IncomeCategory
        if(category instanceof IncomeCategory) {
            ArrayList<Income> castedIncome = (ArrayList<Income>) items;
            try {
                return (ArrayList<T>) castedIncome.stream().filter(anIncome -> anIncome.getCategory().compareTo((IncomeCategory) category) == 0).collect(Collectors.toList());
            } catch (ClassCastException ex) {
                throw new ClassCastException("Can´t use IncomeCategory for Expense´s: " + ex.getMessage());
            }
        }
        //Checks category is s an instance of ExpenseCategory
        else if(category instanceof ExpenseCategory){
            ArrayList<Expense> castedExpenses = (ArrayList<Expense>) items;
            try {
                return (ArrayList<T>) castedExpenses.stream().filter(anExpense -> anExpense.getCategory().compareTo((ExpenseCategory) category) == 0).collect(Collectors.toList());
            } catch (ClassCastException ex){
                throw new ClassCastException("Can´t use ExpenseCategory for Income: " + ex.getMessage());
            }
        } else{
            throw new IllegalArgumentException("The category must be of type IncomeCategory or ExpenseCategory");
        }
    }
}
